@@ -74,6 +74,28 @@ on_aboutbox_response                   (GtkDialog       *dialog,
     gtk_widget_destroy(GTK_WIDGET(dialog));
 }
 
+void
+update_track_offsets (void)
+{
+    GtkTreeIter iter;
+    GtkListStore * store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(lookup_widget(win_main, "tracklist"))));
+    if (store == NULL)
+        return;
+    gboolean rowsleft = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+    int tracknum;
+
+    while (rowsleft)
+    {
+        gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
+            COL_TRACKNUM, &tracknum,
+            -1);
+        gtk_list_store_set(store, &iter,
+            COL_TRACKNUM_VIS, tracknum + global_prefs->first_track_num_offset,
+            -1);
+        rowsleft = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+    }
+}
+
 gboolean
 on_album_artist_focus_out_event        (GtkWidget       *widget,
                                         GdkEventFocus   *event,
@@ -172,6 +194,43 @@ on_album_genre_focus_out_event         (GtkWidget       *widget,
     return FALSE;
 }
 
+// First track number arbitrarily limited to 1001
+gboolean
+on_tracknum_first_focus_out_event      (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data)
+{
+    const gchar * ctext = gtk_entry_get_text(GTK_ENTRY(widget));
+    int first = atoi(ctext);
+
+    if (first >= 1 && first <= 1001)
+    {
+        global_prefs->first_track_num_offset = first - 1;
+    }
+    update_track_offsets();
+
+    char txt[16];
+    snprintf(txt, 16, "%d", global_prefs->first_track_num_offset + 1);
+    gtk_entry_set_text(GTK_ENTRY (widget), txt);
+
+    return FALSE;
+}
+
+void
+on_tracknum_width_changed_event        (GtkComboBox     *combobox,
+                                        gpointer         user_data)
+{
+    gint selected = gtk_combo_box_get_active(combobox);
+    int width = selected + 1;
+
+    if (width >= 1 && width <= 4)
+    {
+        global_prefs->track_num_width = width;
+    }
+
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), global_prefs->track_num_width - 1);
+}
+
 void
 on_artist_edited                    (GtkCellRendererText *cell,
                                      gchar               *path_string,
@@ -262,6 +321,15 @@ on_mp3bitrate_value_changed            (GtkRange        *range,
     gtk_label_set_text(GTK_LABEL(lookup_widget(win_prefs, "bitrate_lbl_2")), bitrate);
 }
 
+void
+on_fdkaac_bitrate_value_changed            (GtkRange        *range,
+                                        gpointer         user_data)
+{
+    char bitrate[8];    
+    snprintf(bitrate, 8, _("%dKbps"), int_to_bitrate((int)gtk_range_get_value(range), 0));
+    gtk_label_set_text(GTK_LABEL(lookup_widget(win_prefs, "fdkaac_bitrate_lbl_2")), bitrate);
+}
+
 void
 on_opusrate_value_changed           (GtkRange   *range,
                                      gpointer   user_data)
@@ -316,6 +384,7 @@ on_prefs_response                      (GtkDialog       *dialog,
         
         get_prefs_from_widgets(global_prefs);
         save_prefs(global_prefs);
+        toggle_allow_tracknum();
     }
     
     gtk_widget_destroy(GTK_WIDGET(dialog));    
@@ -442,8 +511,8 @@ on_edit_split_clicked (void)
                 {
                     *cp = '\0';
                     cp--;
-				}
-				
+                }
+                
                 // Write modified values back into widgets
                 gtk_list_store_set(store, &iter,
                     COL_TRACKARTIST, trackartist,
@@ -451,12 +520,12 @@ on_edit_split_clicked (void)
                     -1);
             }
         }
-		
+        
         if (trackartist)
             free (trackartist);
         if (tracktitle)
             free (tracktitle);
-		
+        
         rowsleft = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
     }
 }
@@ -502,7 +571,6 @@ on_edit_swap_clicked (void)
     }
 }
 
-
 void 
 on_press_f2                       (void)
 {
@@ -592,6 +660,32 @@ on_rip_mp3_toggled                     (GtkToggleButton *togglebutton,
         enable_mp3_widgets();
 }
 
+void
+on_rip_fdkaac_toggled                     (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+    if (gtk_toggle_button_get_active(togglebutton) && !program_exists("fdkaac"))
+    {
+        GtkWidget * dialog;
+        
+        dialog = gtk_message_dialog_new(GTK_WINDOW(win_main), 
+                                        GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
+                                        _("%s was not found in your path. Asunder requires it to create %s files. "
+                                        "All %s functionality is disabled."),
+                                        "'fdkaac'", "AAC", "AAC");
+        gtk_dialog_run(GTK_DIALOG(dialog));
+        gtk_widget_destroy(dialog);
+
+        global_prefs->rip_fdkaac = 0;
+        gtk_toggle_button_set_active(togglebutton, global_prefs->rip_fdkaac);
+    }
+    
+    if (!gtk_toggle_button_get_active(togglebutton))
+        disable_fdkaac_widgets();
+    else
+        enable_fdkaac_widgets();
+}
+
 void
 on_rip_flac_toggled                    (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
@@ -718,31 +812,6 @@ on_rip_monkey_toggled                  (GtkToggleButton *togglebutton,
         enable_monkey_widgets();
 }
 
-void
-on_rip_aac_toggled                  (GtkToggleButton *togglebutton,
-                                     gpointer         user_data)
-{
-    if (gtk_toggle_button_get_active(togglebutton) && !program_exists("neroAacEnc"))
-    {
-        GtkWidget * dialog;
-        dialog = gtk_message_dialog_new(GTK_WINDOW(win_main), 
-                                        GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-                                        _("%s was not found in your path. Asunder requires it to create %s files. "
-                                        "All %s functionality is disabled."),
-                                        "'neroAacEnc'", "MP4", "AAC");
-        gtk_dialog_run(GTK_DIALOG(dialog));
-        gtk_widget_destroy(dialog);
-
-        global_prefs->rip_aac = 0;
-        gtk_toggle_button_set_active(togglebutton, global_prefs->rip_aac);
-    }
-    
-    if (!gtk_toggle_button_get_active(togglebutton))
-        disable_aac_widgets();
-    else
-        enable_aac_widgets();
-}
-
 void
 on_rip_musepack_toggled                  (GtkToggleButton *togglebutton,
                                           gpointer         user_data)
