@@ -9,7 +9,6 @@ the GNU General Public Licence as published by the Free Software
 Foundation; version 2 of the licence.
 
 */
-
 #ifdef HAVE_CONFIG_H
 #    include <config.h>
 #endif
@@ -17,6 +16,7 @@ Foundation; version 2 of the licence.
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 
@@ -111,7 +111,7 @@ create_main (void)
     gtk_container_add (GTK_CONTAINER (toolbar1), about);
     gtk_tool_item_set_is_important (GTK_TOOL_ITEM (about), TRUE);
     
-    table2 = gtk_table_new (3, 3, FALSE);
+    table2 = gtk_table_new (5, 3, FALSE);
     gtk_widget_show (table2);
     gtk_box_pack_start (GTK_BOX (vbox1), table2, FALSE, TRUE, 3);
 
@@ -167,6 +167,51 @@ create_main (void)
                       (GtkAttachOptions) (GTK_FILL),
                       (GtkAttachOptions) (0), 3, 0);
 
+    GtkWidget* tn_hbox = gtk_hbox_new(FALSE, 5);
+    gtk_widget_hide(tn_hbox);
+    gtk_table_attach (GTK_TABLE (table2), tn_hbox, 0, 3, 4, 5,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (0), 3, 0);
+
+    GtkWidget *tn_labelo = gtk_label_new (_("First track number:"));
+    gtk_misc_set_alignment (GTK_MISC (tn_labelo), 0, 0.5);
+    gtk_widget_show (tn_labelo);
+    gtk_box_pack_start(GTK_BOX (tn_hbox), tn_labelo, FALSE, TRUE, 0);
+
+    GtkWidget* tn_first = gtk_entry_new();
+    gtk_widget_show(tn_first);
+    gtk_box_pack_start(GTK_BOX (tn_hbox), tn_first, FALSE, TRUE, 0);
+    gtk_entry_set_width_chars(GTK_ENTRY (tn_first), 4);
+    char txt[16];
+    snprintf(txt, 16, "%d", global_prefs->first_track_num_offset + 1);
+    gtk_entry_set_text(GTK_ENTRY (tn_first), txt);
+
+    GtkWidget* tn_labelw = gtk_label_new (_("Track number width in filename:"));
+    gtk_misc_set_alignment (GTK_MISC (tn_labelw), 0, 0.5);
+    gtk_widget_show (tn_labelw);
+    gtk_box_pack_start(GTK_BOX (tn_hbox), tn_labelw, FALSE, TRUE, 0);
+
+    GtkWidget* tn_width = gtk_combo_box_new();
+    gtk_widget_show(tn_width);
+    gtk_box_pack_start(GTK_BOX (tn_hbox), tn_width, FALSE, TRUE, 0);
+	
+    GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(tn_width), renderer, TRUE);
+    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(tn_width), renderer,
+                                                    "text", 0,
+                                                    NULL);
+    GtkListStore * store = gtk_list_store_new(1, G_TYPE_STRING);
+    GtkTreeIter iter;
+    for (int i = 1; i <= 4; ++i) {
+        char buff[2];
+        snprintf(buff, 2, "%d", i);
+        gtk_list_store_append(store, &iter);
+        gtk_list_store_set(store, &iter, 0, buff, -1);
+    }
+    gtk_combo_box_set_model(GTK_COMBO_BOX(tn_width), GTK_TREE_MODEL(store));
+    g_object_unref(store);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(tn_width), global_prefs->track_num_width - 1);
+	
     genre_label	= gtk_label_new (_("Genre / Year:"));								// lnr
     gtk_misc_set_alignment (GTK_MISC ( genre_label ), 0, 0);
     gtk_widget_show (genre_label);
@@ -263,6 +308,12 @@ create_main (void)
     g_signal_connect ((gpointer) album_year, "focus_out_event",
                                         G_CALLBACK (on_year_focus_out_event),
                                         NULL);
+    g_signal_connect ((gpointer) tn_first, "focus_out_event",
+                                        G_CALLBACK (on_tracknum_first_focus_out_event),
+                                        NULL);
+    g_signal_connect ((gpointer) tn_width, "changed",
+                                        G_CALLBACK (on_tracknum_width_changed_event),
+                                        NULL);
     
     /* KEYBOARD accelerators */
     GtkAccelGroup* accelGroup;
@@ -313,6 +364,9 @@ create_main (void)
     GLADE_HOOKUP_OBJECT (main_win, album_genre, "album_genre");			// lnr
     GLADE_HOOKUP_OBJECT (main_win, genre_label, "genre_label" );		// lnr
     GLADE_HOOKUP_OBJECT (main_win, album_year, "album_year");
+    GLADE_HOOKUP_OBJECT (main_win, tn_hbox, "tn_hbox");
+    GLADE_HOOKUP_OBJECT (main_win, tn_first, "tn_first");
+    GLADE_HOOKUP_OBJECT (main_win, tn_width, "tn_width");
     
     return main_win;
 }
@@ -340,6 +394,11 @@ create_prefs (void)
     GtkWidget *mp3_vbr;
     GtkWidget *hbox9;
     GtkWidget *rip_mp3;
+    GtkWidget *vbox2X;
+    GtkWidget *frame3X;
+    GtkWidget *alignment8X;
+    GtkWidget *hbox9X;
+    GtkWidget *rip_fdkaac;
     GtkWidget *frame4;
     GtkWidget *alignment9;
     GtkWidget *hbox10;
@@ -418,14 +477,14 @@ create_prefs (void)
     /* END GENERAL tab */
     
     /* FILENAMES tab */
-    vbox = gtk_vbox_new (FALSE, 5);
-    gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-    gtk_widget_show (vbox);
-    gtk_container_add (GTK_CONTAINER (notebook1), vbox);
+    vbox2 = gtk_vbox_new (FALSE, 5);
+    gtk_container_set_border_width (GTK_CONTAINER (vbox2), 5);
+    gtk_widget_show (vbox2);
+    gtk_container_add (GTK_CONTAINER (notebook1), vbox2);
 
     frame2 = gtk_frame_new (NULL);
     gtk_widget_show (frame2);
-    gtk_box_pack_start (GTK_BOX (vbox), frame2, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox2), frame2, FALSE, FALSE, 0);
     
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
@@ -513,6 +572,11 @@ create_prefs (void)
     gtk_frame_set_label_widget (GTK_FRAME (frame2), label);
     gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 
+    GtkWidget * allow_tracknum = gtk_check_button_new_with_label (_("Allow changing first track number"));
+    gtk_widget_show (allow_tracknum);
+    gtk_box_pack_start (GTK_BOX (vbox2), allow_tracknum, FALSE, FALSE, 5);
+    GLADE_HOOKUP_OBJECT (prefs, allow_tracknum, "allow_tracknum");
+
     label = gtk_label_new (_("Filenames"));
     gtk_widget_show (label);
     gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label);
@@ -520,6 +584,7 @@ create_prefs (void)
     
     /* ENCODE tab */
     GtkWidget *mp3bitrate;
+    GtkWidget *fdkaac_bitrate;
     GtkWidget *oggLbl;
     GtkWidget *oggquality;
     GtkWidget *rip_ogg;
@@ -612,7 +677,7 @@ create_prefs (void)
                                         G_CALLBACK (on_rip_mp3_toggled),
                                         NULL);
     /* END MP3 */
-    
+	
     /* OGG */
     frame4 = gtk_frame_new (NULL);
     gtk_frame_set_shadow_type(GTK_FRAME(frame4), GTK_SHADOW_IN);
@@ -649,6 +714,57 @@ create_prefs (void)
                                         NULL);
     /* END OGG */
 
+    /* FDK-AAC  */
+    frame3X = gtk_frame_new (NULL);
+    gtk_frame_set_shadow_type(GTK_FRAME(frame3X), GTK_SHADOW_IN);
+    gtk_widget_show (frame3X);
+    gtk_box_pack_start (GTK_BOX (vbox), frame3X, FALSE, FALSE, 0);
+
+    alignment8X = gtk_alignment_new (0.5, 0.5, 1, 1);
+    gtk_widget_show (alignment8X);
+    gtk_container_add (GTK_CONTAINER (frame3X), alignment8X);
+    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment8X), 2, 2, 12, 2);
+    
+    vbox2X = gtk_vbox_new (FALSE, 0);
+    gtk_widget_show (vbox2X);
+    gtk_container_add (GTK_CONTAINER (alignment8X), vbox2X);
+
+    hbox9X = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox9X);
+    gtk_box_pack_start (GTK_BOX (vbox2X), hbox9X, TRUE, TRUE, 0);    
+
+    label = gtk_label_new (_("Bitrate"));
+    gtk_widget_show (label);
+    gtk_box_pack_start (GTK_BOX (hbox9X), label, FALSE, FALSE, 0);
+    GLADE_HOOKUP_OBJECT (prefs, label, "fdkaac_bitrate_lbl");
+    
+    fdkaac_bitrate = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 14, 1, 1, 1)));
+    gtk_widget_show (fdkaac_bitrate);
+    gtk_box_pack_start (GTK_BOX (hbox9X), fdkaac_bitrate, TRUE, TRUE, 5);
+    gtk_scale_set_draw_value (GTK_SCALE (fdkaac_bitrate), FALSE);
+    gtk_scale_set_digits (GTK_SCALE (fdkaac_bitrate), 0);
+    g_signal_connect ((gpointer) fdkaac_bitrate, "value_changed",
+                                        G_CALLBACK (on_fdkaac_bitrate_value_changed),
+                                        NULL);
+    
+    tooltips = gtk_tooltips_new ();
+    gtk_tooltips_set_tip (tooltips, fdkaac_bitrate, _("Higher bitrate is better quality but also bigger file. Most people use 192Kbps."), NULL);
+    
+    char kbps_textX[10];
+    snprintf(kbps_textX, 10, _("%dKbps"), 32);
+    label = gtk_label_new (kbps_textX);
+    gtk_widget_show (label);
+    gtk_box_pack_start (GTK_BOX (hbox9X), label, FALSE, FALSE, 0);
+    GLADE_HOOKUP_OBJECT (prefs, label, "fdkaac_bitrate_lbl_2");
+    
+    rip_fdkaac = gtk_check_button_new_with_mnemonic (_("AAC (lossy compression)"));
+    gtk_widget_show (rip_fdkaac);
+    gtk_frame_set_label_widget (GTK_FRAME (frame3X), rip_fdkaac);
+    g_signal_connect ((gpointer) rip_fdkaac, "toggled",
+                                        G_CALLBACK (on_rip_fdkaac_toggled),
+                                        NULL);
+    /* END FDK-AAC */
+    
     /* FLAC */
     frame5 = gtk_frame_new (NULL);
     gtk_frame_set_shadow_type(GTK_FRAME(frame5), GTK_SHADOW_IN);
@@ -943,61 +1059,14 @@ create_prefs (void)
     GLADE_HOOKUP_OBJECT (prefs, rip_monkey, "rip_monkey");
     /* END MONKEY */
     
-    expander = gtk_expander_new(_("Proprietary encoders"));
-    gtk_widget_show (expander);
-    gtk_box_pack_start (GTK_BOX (vbox), expander, FALSE, FALSE, 0);
-    GLADE_HOOKUP_OBJECT (prefs, expander, "proprietary_formats_expander");
+    //~ expander = gtk_expander_new(_("Proprietary encoders"));
+    //~ gtk_widget_show (expander);
+    //~ gtk_box_pack_start (GTK_BOX (vbox), expander, FALSE, FALSE, 0);
+    //~ GLADE_HOOKUP_OBJECT (prefs, expander, "proprietary_formats_expander");
     
-    hiddenbox = gtk_vbox_new (FALSE, 0);
-    gtk_widget_show (hiddenbox);
-    gtk_container_add (GTK_CONTAINER (expander), hiddenbox);
-    
-    /* AAC */
-    GtkWidget* rip_aac;
-    GtkWidget* aacQuality;
-    GtkWidget* aacVbox;
-    
-    frame8 = gtk_frame_new (NULL);
-    gtk_frame_set_shadow_type(GTK_FRAME(frame8), GTK_SHADOW_IN);
-    gtk_widget_show (frame8);
-    gtk_box_pack_start (GTK_BOX (hiddenbox), frame8, FALSE, FALSE, 0);
-    
-    alignment11 = gtk_alignment_new (0.5, 0.5, 1, 1);
-    gtk_widget_show (alignment11);
-    gtk_container_add (GTK_CONTAINER (frame8), alignment11);
-    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment11), 2, 2, 12, 2);
-    
-    aacVbox = gtk_vbox_new (FALSE, 0);
-    gtk_widget_show (aacVbox);
-    gtk_container_add (GTK_CONTAINER (alignment11), aacVbox);
-    
-    hbox13 = gtk_hbox_new (FALSE, 0);
-    gtk_widget_show (hbox13);
-    gtk_box_pack_start (GTK_BOX (aacVbox), hbox13, FALSE, FALSE, 0);
-    
-    label = gtk_label_new (_("Quality"));
-    gtk_widget_show (label);
-    gtk_box_pack_start (GTK_BOX (hbox13), label, FALSE, FALSE, 0);
-    GLADE_HOOKUP_OBJECT (prefs, label, "aac_quality_lbl");
-    
-    aacQuality = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 1, 100, 1, 1, 1)));
-    gtk_widget_show (aacQuality);
-    gtk_box_pack_start (GTK_BOX (hbox13), aacQuality, TRUE, TRUE, 5);
-    gtk_scale_set_value_pos (GTK_SCALE (aacQuality), GTK_POS_RIGHT);
-    gtk_scale_set_digits (GTK_SCALE (aacQuality), 0);
-    GLADE_HOOKUP_OBJECT (prefs, aacQuality, "aac_quality_slider");
-    
-    tooltips = gtk_tooltips_new ();
-    gtk_tooltips_set_tip (tooltips, aacQuality, _("Higher quality means bigger file. Default is 60."), NULL);
-    
-    rip_aac = gtk_check_button_new_with_mnemonic (_("AAC (lossy compression, Nero encoder)"));
-    gtk_widget_show (rip_aac);
-    gtk_frame_set_label_widget (GTK_FRAME (frame8), rip_aac);
-    g_signal_connect ((gpointer) rip_aac, "toggled",
-                                        G_CALLBACK (on_rip_aac_toggled),
-                                        NULL);
-    GLADE_HOOKUP_OBJECT (prefs, rip_aac, "rip_aac");
-    /* END AAC */
+    //~ hiddenbox = gtk_vbox_new (FALSE, 0);
+    //~ gtk_widget_show (hiddenbox);
+    //~ gtk_container_add (GTK_CONTAINER (expander), hiddenbox);
     
     label = gtk_label_new (_("Encode"));
     gtk_widget_show (label);
@@ -1165,6 +1234,8 @@ create_prefs (void)
     GLADE_HOOKUP_OBJECT (prefs, oggLbl, "ogg_lbl");
     GLADE_HOOKUP_OBJECT (prefs, oggquality, "oggquality");
     GLADE_HOOKUP_OBJECT (prefs, rip_ogg, "rip_ogg");
+    GLADE_HOOKUP_OBJECT (prefs, rip_fdkaac, "rip_fdkaac");
+    GLADE_HOOKUP_OBJECT (prefs, fdkaac_bitrate, "fdkaac_bitrate");
     GLADE_HOOKUP_OBJECT (prefs, flacLbl, "flac_lbl");
     GLADE_HOOKUP_OBJECT (prefs, flaccompression, "flaccompression");
     GLADE_HOOKUP_OBJECT (prefs, rip_flac, "rip_flac");
@@ -1324,6 +1395,20 @@ void enable_mp3_widgets(void)
     gtk_widget_set_sensitive(lookup_widget(win_prefs, "bitrate_lbl_2"), TRUE);
 }
 
+void disable_fdkaac_widgets(void)
+{
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "fdkaac_bitrate_lbl"), FALSE);
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "fdkaac_bitrate"), FALSE);
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "fdkaac_bitrate_lbl_2"), FALSE);
+}
+
+void enable_fdkaac_widgets(void)
+{
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "fdkaac_bitrate_lbl"), TRUE);
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "fdkaac_bitrate"), TRUE);
+    gtk_widget_set_sensitive(lookup_widget(win_prefs, "fdkaac_bitrate_lbl_2"), TRUE);
+}
+
 void disable_ogg_widgets(void)
 {
     gtk_widget_set_sensitive(lookup_widget(win_prefs, "ogg_lbl"), FALSE);
@@ -1400,18 +1485,6 @@ void enable_monkey_widgets(void)
     gtk_widget_set_sensitive(lookup_widget(win_prefs, "monkey_compression_slider"), TRUE);
 }
 
-void disable_aac_widgets(void)
-{
-    gtk_widget_set_sensitive(lookup_widget(win_prefs, "aac_quality_lbl"), FALSE);
-    gtk_widget_set_sensitive(lookup_widget(win_prefs, "aac_quality_slider"), FALSE);
-}
-
-void enable_aac_widgets(void)
-{
-    gtk_widget_set_sensitive(lookup_widget(win_prefs, "aac_quality_lbl"), TRUE);
-    gtk_widget_set_sensitive(lookup_widget(win_prefs, "aac_quality_slider"), TRUE);
-}
-
 void disable_musepack_widgets(void)
 {
     gtk_widget_set_sensitive(lookup_widget(win_prefs, "musepack_bitrate_lbl"), FALSE);
@@ -1424,8 +1497,34 @@ void enable_musepack_widgets(void)
     gtk_widget_set_sensitive(lookup_widget(win_prefs, "musepack_bitrate_slider"), TRUE);
 }
 
+void
+toggle_allow_tracknum(void)
+{
+    // "Toggle Track Offset/Width control"
+    GtkWidget* tn_hbox = lookup_widget(win_main, "tn_hbox");
+    GtkWidget* tn_first = lookup_widget(win_main, "tn_first");
+    GtkWidget* tn_width = lookup_widget(win_main, "tn_width");
+    gboolean active = global_prefs->allow_first_track_num_change;
+
+    if (!active)
+    {
+        global_prefs->first_track_num_offset = 0;
+        global_prefs->track_num_width = 2;
+        gtk_widget_hide(tn_hbox);
+    }
+    else
+    {
+        char txt[16];
+        snprintf(txt, 16, "%d", global_prefs->first_track_num_offset + 1);
+        gtk_entry_set_text(GTK_ENTRY (tn_first), txt);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(tn_width), global_prefs->track_num_width - 1);
+        gtk_widget_show(tn_hbox);
+    }
+    update_track_offsets();
+}
+
 const char* 
-GBLprogramName = "Asunder 2.9.2";
+GBLprogramName = "Asunder 2.9.3";
 
 static const char* 
 GBLauthors[2] = {
@@ -1433,7 +1532,7 @@ GBLauthors[2] = {
 "\n"
 "Andrew Smith\n"
 "http://littlesvr.ca/contact.php\n"
-"Summer 2005 - 2009\n"
+"2005 - 2018\n"
 "- maintainer\n"
 "\n"
 "Eric Lathrop\n"
@@ -1457,6 +1556,7 @@ GBLauthors[2] = {
 "- Remove 'single genre' code that's been unused since 2010.\n"
 "- Added features to automatically fix some common mistakes in CDDB entries.\n"
 "- Fixed the wavpack and mpc encoders to include APEv2 metadata.\n"
+"- Allow changing the first track number and the width of the track number in the filename.\n"
 "(A few patches were mistakenly attributed to \"Gregory Montego\" and \"Gergory Margo\")\n"
 "\n"
 "Andreas Ronnquist\n"
@@ -1492,6 +1592,7 @@ GBLauthors[2] = {
 "- Support for autocompletion in artist/album/genre fields.\n"
 "\n"
 "Jonathan 'theJPster' Pallant\n"
+"- Move from neroAacEnc to fdkaac.\n"
 "- Tag AAC files using neroAacTag.\n"
 "\n"
 "Micah Lindstrom\n"
