@@ -92,13 +92,13 @@ void abort_threads()
         kill(pid, SIGKILL);
     if ((pid = musepack_pid)   != 0)
         kill(pid, SIGKILL);
-    if ((pid = aac_pid)        != 0)
+    if ((pid = fdkaac_pid)     != 0)
         kill(pid, SIGKILL);
     
     /* wait until all the worker threads are done */
     while (cdparanoia_pid != 0 || lame_pid != 0 || oggenc_pid != 0 || 
            opusenc_pid != 0 || flac_pid != 0 || wavpack_pid != 0 || monkey_pid != 0 ||
-           musepack_pid != 0 || aac_pid != 0)
+           musepack_pid != 0 || fdkaac_pid != 0)
     {
         debugLog("w1");
         usleep(100000);
@@ -178,7 +178,7 @@ void dorip()
     // make sure there's at least one format to rip to
     if (!global_prefs->rip_wav && !global_prefs->rip_mp3 && !global_prefs->rip_ogg && !global_prefs->rip_opus &&
         !global_prefs->rip_flac && !global_prefs->rip_wavpack && !global_prefs->rip_monkey &&
-        !global_prefs->rip_musepack && !global_prefs->rip_aac)
+        !global_prefs->rip_musepack && !global_prefs->rip_fdkaac)
     {
         GtkWidget * dialog;
         dialog = gtk_message_dialog_new(GTK_WINDOW(win_main), 
@@ -327,7 +327,7 @@ void dorip()
             
             free(filename);
         }
-        if (global_prefs->rip_aac)
+        if (global_prefs->rip_fdkaac)
         {
             char * filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, playlist, "m4a.m3u");
             
@@ -399,6 +399,7 @@ gpointer rip(gpointer data)
     {
         int riptrack;
         int tracknum;
+        int tracknum_vis;
         char * trackartist;
         char * tracktitle;
 
@@ -406,6 +407,7 @@ gpointer rip(gpointer data)
             gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
                 COL_RIPTRACK, &riptrack,
                 COL_TRACKNUM, &tracknum,
+                COL_TRACKNUM_VIS, &tracknum_vis,
                 COL_TRACKARTIST, &trackartist,
                 COL_TRACKTITLE, &tracktitle,
                 -1);
@@ -435,7 +437,7 @@ gpointer rip(gpointer data)
             //~musicfilename = parse_format(global_prefs->format_music, tracknum, trackyear, trackartist, albumtitle, tracktitle);
             //~musicfilename = parse_format(global_prefs->format_music, tracknum, albumyear, trackartist, albumtitle, albumgenre, tracktitle);            
             albumdir = parse_format(global_prefs->format_albumdir, 0, albumyear, albumartist_trimmed, albumtitle_trimmed, albumgenre_trimmed, NULL);
-            musicfilename = parse_format(global_prefs->format_music, tracknum, albumyear, trackartist_trimmed, albumtitle_trimmed, albumgenre_trimmed, tracktitle_trimmed);
+            musicfilename = parse_format(global_prefs->format_music, tracknum_vis, albumyear, trackartist_trimmed, albumtitle_trimmed, albumgenre_trimmed, tracktitle_trimmed);
             wavfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wav");
             
             snprintf(logStr, 1024, "Ripping track %d to \"%s\"\n", tracknum, wavfilename);
@@ -605,6 +607,7 @@ gpointer encode(gpointer data)
     {
         int riptrack;
         int tracknum;
+        int tracknum_vis;
         char* trackartist;
         char* tracktitle;
         char* tracktime;
@@ -641,6 +644,7 @@ gpointer encode(gpointer data)
             gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
                 COL_RIPTRACK, &riptrack,
                 COL_TRACKNUM, &tracknum,
+                COL_TRACKNUM_VIS, &tracknum_vis,
                 COL_TRACKARTIST, &trackartist,
                 COL_TRACKTITLE, &tracktitle,
                 COL_TRACKTIME, &tracktime,
@@ -672,7 +676,7 @@ gpointer encode(gpointer data)
             //~musicfilename = parse_format(global_prefs->format_music, tracknum, trackyear, trackartist, album_title, tracktitle);
             //~musicfilename = parse_format(global_prefs->format_music, tracknum, album_year, trackartist, album_title, genre, tracktitle);
             albumdir = parse_format(global_prefs->format_albumdir, 0, album_year, album_artist_trimmed, album_title_trimmed, album_genre_trimmed, NULL);
-            musicfilename = parse_format(global_prefs->format_music, tracknum, album_year, trackartist_trimmed, album_title_trimmed, album_genre_trimmed, tracktitle_trimmed);
+            musicfilename = parse_format(global_prefs->format_music, tracknum_vis, album_year, trackartist_trimmed, album_title_trimmed, album_genre_trimmed, tracktitle_trimmed);
             
             wavfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wav");
 
@@ -722,7 +726,7 @@ gpointer encode(gpointer data)
                 if(doEncode)
                     //~ lame(tracknum, trackartist, album_title, tracktitle, genre, trackyear, wavfilename, mp3filename, 
                          //~ global_prefs->mp3_vbr, global_prefs->mp3_bitrate, &mp3_percent);
-                    lame(tracknum, trackartist, album_title, tracktitle, album_genre, album_year, wavfilename, mp3filename,
+                    lame(tracknum_vis, trackartist, album_title, tracktitle, album_genre, album_year, wavfilename, mp3filename,
                          global_prefs->mp3_vbr, global_prefs->mp3_bitrate, &mp3_percent);
 
                 if (playlist_mp3)
@@ -778,7 +782,7 @@ gpointer encode(gpointer data)
                 
                 if(doEncode)
                 {
-                    oggenc(tracknum, trackartist, album_title, tracktitle, album_year, album_genre, wavfilename,
+                    oggenc(tracknum_vis, trackartist, album_title, tracktitle, album_year, album_genre, wavfilename,
                            oggfilename, global_prefs->ogg_quality, &ogg_percent);
                 }
 
@@ -834,7 +838,7 @@ gpointer encode(gpointer data)
                     doEncode = true;
 
                 if(doEncode)
-                    opusenc(tracknum, trackartist, album_title, tracktitle, album_year, album_genre, wavfilename,
+                    opusenc(tracknum_vis, trackartist, album_title, tracktitle, album_year, album_genre, wavfilename,
                            opusfilename, global_prefs->opus_bitrate, &opus_percent);
 
                 if (playlist_opus)
@@ -891,7 +895,7 @@ gpointer encode(gpointer data)
                 if(doEncode)
                     //~ flac(tracknum, trackartist, album_title, tracktitle, genre, trackyear, wavfilename, 
                          //~ flacfilename, global_prefs->flac_compression, &flac_percent);
-                    flac(tracknum, trackartist, album_artist, single_artist, album_title, tracktitle, album_genre, album_year, wavfilename,
+                    flac(tracknum_vis, trackartist, album_artist, single_artist, album_title, tracktitle, album_genre, album_year, wavfilename,
                          flacfilename, global_prefs->flac_compression, &flac_percent); //mw
 
                 
@@ -956,7 +960,7 @@ gpointer encode(gpointer data)
                 
                 if(doEncode)
                 {
-                    wavpack(tracknum, trackartist, album_title, tracktitle, album_year, album_genre,
+                    wavpack(tracknum_vis, trackartist, album_title, tracktitle, album_year, album_genre,
                             wavfilename, wavpackfilename, wavpackfilename2,
                             global_prefs->wavpack_compression, global_prefs->wavpack_hybrid, 
                             int_to_wavpack_bitrate(global_prefs->wavpack_bitrate), &wavpack_percent);
@@ -1074,7 +1078,7 @@ gpointer encode(gpointer data)
                 
                 if(doEncode)
                 {
-                    musepack(tracknum, trackartist, album_title, tracktitle, album_year, album_genre,
+                    musepack(tracknum_vis, trackartist, album_title, tracktitle, album_year, album_genre,
                              wavfilename, musepackfilename,
                              int_to_musepack_int(global_prefs->musepack_bitrate), 
                              &musepack_percent);
@@ -1088,7 +1092,7 @@ gpointer encode(gpointer data)
                 }
                 free(musepackfilename);
             }
-            if (global_prefs->rip_aac)
+            if (global_prefs->rip_fdkaac)
             {
                 if (aborted)
                 {
@@ -1113,7 +1117,7 @@ gpointer encode(gpointer data)
                     close_playlists();
                     return NULL;
                 }
-
+				
                 char * aacfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "m4a");
                 snprintf(logStr, 1024, "Encoding track %d to \"%s\"\n", tracknum, aacfilename);
                 debugLog(logStr);
@@ -1133,10 +1137,10 @@ gpointer encode(gpointer data)
                 
                 if(doEncode)
                 {
-                    aac(tracknum, trackartist, album_title, tracktitle, album_genre, album_year,
-                        wavfilename, aacfilename,
-                        global_prefs->aac_quality, 
-                        &aac_percent);
+                    fdkaac(tracknum, trackartist, album_title, tracktitle, album_genre, album_year,
+                           wavfilename, aacfilename,
+                           global_prefs->fdkaac_bitrate,
+                           &aac_percent);
                 }
                 
                 if (playlist_aac)
@@ -1224,7 +1228,7 @@ gpointer encode(gpointer data)
     /* wait until all the worker threads are done */
     while (cdparanoia_pid != 0 || lame_pid != 0 || oggenc_pid != 0 || 
            opusenc_pid != 0 || flac_pid != 0 || wavpack_pid != 0 || monkey_pid != 0 ||
-           musepack_pid != 0 || aac_pid != 0)
+           musepack_pid != 0 || fdkaac_pid != 0)
     {
         debugLog("w2");
         usleep(100000);
@@ -1269,7 +1273,7 @@ gpointer track(gpointer data)
         parts++;
     if(global_prefs->rip_musepack) 
         parts++;
-    if(global_prefs->rip_aac) 
+    if(global_prefs->rip_fdkaac)
         parts++;
     
     gdk_threads_enter();
