@@ -29,15 +29,15 @@ Foundation; version 2 of the licence.
 #include "prefs.h"
 #include "util.h"
 
-pid_t cdparanoia_pid;
-pid_t lame_pid;
-pid_t oggenc_pid;
-pid_t opusenc_pid;
-pid_t flac_pid;
-pid_t wavpack_pid;
-pid_t monkey_pid;
-pid_t musepack_pid;
-pid_t aac_pid;
+pid_t cdparanoia_pid = 0;
+pid_t lame_pid = 0;
+pid_t oggenc_pid = 0;
+pid_t opusenc_pid = 0;
+pid_t flac_pid = 0;
+pid_t wavpack_pid = 0;
+pid_t monkey_pid = 0;
+pid_t musepack_pid = 0;
+pid_t fdkaac_pid = 0;
 
 int numCdparanoiaFailed;
 int numLameFailed;
@@ -87,18 +87,8 @@ void unblockSigChld(void)
     waitBeforeSigchld = false;
 }
 
-extern pid_t cdparanoia_pid;
-extern pid_t lame_pid;
-extern pid_t oggenc_pid;
-extern pid_t opusenc_pid;
-extern pid_t flac_pid;
-extern pid_t wavpack_pid;
-extern pid_t monkey_pid;
-extern pid_t musepack_pid;
-extern pid_t aac_pid;
-
 // Signal handler to find out when our child has exited.
-// Do not pot any printf or syslog into here, it causes a deadlock.
+// Do not put any printf or syslog into here, it causes a deadlock.
 //
 void sigchld(int signum)
 {
@@ -154,9 +144,9 @@ void sigchld(int signum)
             musepack_pid = 0;
             numMusepackFailed++;
         }
-        else if (pid == aac_pid)
+        else if (pid == fdkaac_pid)
         {
-            aac_pid = 0;
+            fdkaac_pid = 0;
             numAacFailed++;
         }
     }
@@ -203,9 +193,9 @@ void sigchld(int signum)
             musepack_pid = 0;
             numMusepackOk++;
         }
-        else if (pid == aac_pid)
+        else if (pid == fdkaac_pid)
         {
-            aac_pid = 0;
+            fdkaac_pid = 0;
             numAacOk++;
         }
     }
@@ -437,14 +427,12 @@ void lame(int tracknum,
     int sector;
     int end;
     
-    char tracknum_text[4];
+    char * tracknum_text = NULL;
     char bitrate_text[4];
     const char * args[19];
 
 //    fprintf( stderr, " lame()   Genre: %s Artist: %s Title: %s\n", genre, artist, title );	// lnr
 
-    snprintf(tracknum_text, 4, "%d", tracknum);
-    
     pos = 0;
     args[pos++] = "lame";
     if (vbr)
@@ -457,7 +445,7 @@ void lame(int tracknum,
     }
     args[pos++] = bitrate_text;
     args[pos++] = "--id3v2-only";
-    if ((tracknum > 0) && (tracknum < 100))
+    if (tracknum > 0 && asprintf(&tracknum_text, "%d", tracknum) > 0)
     {
         args[pos++] = "--tn";
         args[pos++] = tracknum_text;
@@ -499,6 +487,7 @@ void lame(int tracknum,
     args[pos++] = NULL;
 
     fd = exec_with_output(args, STDERR_FILENO, &lame_pid, NULL);
+    free(tracknum_text);
     
     do
     {
@@ -568,11 +557,10 @@ void oggenc(int tracknum,
     int sector;
     int end;
 
-    char tracknum_text[4];
+    char * tracknum_text = NULL;
     char quality_level_text[3];
     const char * args[19];
 
-    snprintf(tracknum_text, 4, "%d", tracknum);
     snprintf(quality_level_text, 3, "%d", quality_level);
     
     pos = 0;
@@ -580,7 +568,7 @@ void oggenc(int tracknum,
     args[pos++] = "-q";
     args[pos++] = quality_level_text;
     
-    if ((tracknum > 0) && (tracknum < 100))
+    if (tracknum > 0 && asprintf(&tracknum_text, "%d", tracknum) > 0)
     {
         args[pos++] = "-N";
         args[pos++] = tracknum_text;
@@ -616,6 +604,7 @@ void oggenc(int tracknum,
     args[pos++] = NULL;
     
     fd = exec_with_output(args, STDERR_FILENO, &oggenc_pid, NULL);
+    free(tracknum_text);
     
     do
     {
@@ -684,7 +673,7 @@ void opusenc(int tracknum,
 
     int pos;
     char bitrate_text[4];
-    char tracknum_text[16];
+    char * tracknum_text = NULL;
     char album_text[128];
     char year_text[32];
     char genre_text[64];
@@ -697,9 +686,8 @@ void opusenc(int tracknum,
     args[pos++] = "--bitrate";
     args[pos++] = bitrate_text;
 
-    if ((tracknum > 0) && (tracknum < 100))
+    if (tracknum > 0 && asprintf(&tracknum_text, "TRACKNUMBER=%d", tracknum) > 0)
     {
-        snprintf(tracknum_text,16,"TRACKNUMBER=%d",tracknum);
         args[pos++] = "--comment";
         args[pos++] = tracknum_text;
     }
@@ -736,6 +724,7 @@ void opusenc(int tracknum,
     args[pos++] = NULL;
 
     fd = exec_with_output(args, STDERR_FILENO, &oggenc_pid, NULL);
+    free(tracknum_text);
 
     int opussize;
     char opusbuf[256];
@@ -794,7 +783,7 @@ void flac(int tracknum,
     
     int sector;
     
-    char tracknum_text[16];
+    char * tracknum_text = NULL;
     char * artist_text = NULL;
     char * albumartist_text = NULL; //mw
     char * album_text = NULL;
@@ -804,8 +793,6 @@ void flac(int tracknum,
     char compression_level_text[3];
     const char * args[21];
     
-    snprintf(tracknum_text, 16, "TRACKNUMBER=%d", tracknum);
-    
     if(artist != NULL)
     {
         artist_text = malloc(strlen(artist) + 8);
@@ -862,7 +849,7 @@ void flac(int tracknum,
     args[pos++] = "flac";
     args[pos++] = "-f";
     args[pos++] = compression_level_text;
-    if ((tracknum > 0) && (tracknum < 100))
+    if (tracknum > 0 && asprintf(&tracknum_text, "TRACKNUMBER=%d", tracknum) > 0)
     {
         args[pos++] = "-T";
         args[pos++] = tracknum_text;
@@ -908,6 +895,7 @@ void flac(int tracknum,
     
     fd = exec_with_output(args, STDERR_FILENO, &flac_pid, NULL);
     
+    free(tracknum_text);
     free(artist_text);
     free(album_text);
     free(albumartist_text);
@@ -996,10 +984,10 @@ void wavpack(int tracknum,
     // target directory and temporary names for the encoded files, then rename
     // them afterwards.
 
-    char trackname_wv[16];
-    char trackname_wvc[16];
-    snprintf(trackname_wv, sizeof(trackname_wv), "wv%02d.wv", tracknum);
-    snprintf(trackname_wvc, sizeof(trackname_wvc), "wv%02d.wvc", tracknum);
+    char * trackname_wv = NULL;
+    char * trackname_wvc = NULL;
+    size = asprintf(&trackname_wv, "wv%02d.wv", tracknum);
+    size = asprintf(&trackname_wvc, "wv%02d.wvc", tracknum);
     gchar * dir = g_path_get_dirname(wavpackfilename_wv);
     gchar * xwavpackfilename_wv = g_build_filename(dir, trackname_wv, NULL);
     gchar * xwavpackfilename_wvc = g_build_filename(dir, trackname_wvc, NULL);
@@ -1029,7 +1017,7 @@ void wavpack(int tracknum,
 
     // Tags
 
-    int tracknum_len = sizeof("TRACK=")  + 2;  // sizeof includes terminating '\0'
+    int tracknum_len = snprintf(NULL, 0, "TRACK=%d", tracknum) + 1;   // sizeof includes terminating '\0', snprintf does not.
     int artist_len   = sizeof("ARTIST=") + (artist ? strlen(artist) : 0);
     int album_len    = sizeof("ALBUM=")  + (album  ? strlen(album)  : 0);
     int title_len    = sizeof("TITLE=")  + (title  ? strlen(title)  : 0);
@@ -1045,7 +1033,7 @@ void wavpack(int tracknum,
     if (!tracknum_buf || !artist_buf || !album_buf || !title_buf || !year_buf || !genre_buf)
         fatalError("wavpack: malloc failed. Out of memory.");
 
-    if (tracknum > 0 && tracknum < 100)
+    if (tracknum > 0)
     {
         snprintf(tracknum_buf, tracknum_len, "TRACK=%d", tracknum);
         args[pos++] = "-w";
@@ -1167,6 +1155,8 @@ void wavpack(int tracknum,
     free(title_buf);
     free(year_buf);
     free(genre_buf);
+    free(trackname_wv);
+    free(trackname_wvc);
     *progress = 1;
 }
 
@@ -1264,10 +1254,9 @@ void musepack(int tracknum,
     args[pos++] = qualityParam;
 
     // Tags
-    char track[4];
-    if (tracknum > 0 && tracknum < 100)
+    char * track = NULL;
+    if (tracknum > 0 && asprintf(&track, "%d", tracknum) > 0)
     {
-        snprintf(track, 4, "%d", tracknum);
         args[pos++] = "--track";
         args[pos++] = track;
     }
@@ -1302,6 +1291,7 @@ void musepack(int tracknum,
     args[pos++] = NULL;
     
     fd = exec_with_output(args, STDERR_FILENO, &musepack_pid, NULL);
+    free(track);
     
     int size;
     char buf[256];
@@ -1344,7 +1334,7 @@ void musepack(int tracknum,
     *progress = 1;
 }
 
-void aac(int tracknum,
+void fdkaac(int tracknum,
          const char * artist,
          const char * album,
          const char * title,
@@ -1352,111 +1342,118 @@ void aac(int tracknum,
          const char * year,
          const char* wavfilename,
          const char* aacfilename,
-         int quality,
+         int bitrate,
          double* progress)
 {
-    const char* args[9];
-    char* dynamic_args[6];
+    const char* args[21];
+    char bitrate_text[4];
     int fd;
     int pos;
-    int dyn_pos;
-    
+
     pos = 0;
-    args[pos++] = "neroAacEnc";
-    
-    args[pos++] = "-q";
-    char qualityParam[5];
-    snprintf(qualityParam, 5, "0.%d", quality);
-    args[pos++] = qualityParam;
-    
-    args[pos++] = "-if";
-    args[pos++] = wavfilename;
-    args[pos++] = "-of";
+    args[pos++] = "fdkaac";
+
+    /* fdkaac has a VBR option but it's unsupported */
+    args[pos++] = "-m";
+    args[pos++] = "0";
+    args[pos++] = "-b";
+    snprintf(bitrate_text, 4, "%d", int_to_bitrate(bitrate, 0));
+    args[pos++] = bitrate_text;
+
+
+    char * track = NULL;
+    if (tracknum > 0 && asprintf(&track, "%d", tracknum) > 0)
+    {
+        args[pos++] = "--track";
+        args[pos++] = track;
+    }
+
+    if ((title != NULL) && (strlen(title) > 0))
+    {
+        args[pos++] = "--title";
+        args[pos++] = title;
+    }
+
+    if ((artist != NULL) && (strlen(artist) > 0))
+    {
+        args[pos++] = "--artist";
+        args[pos++] = artist;
+    }
+
+    if ((album != NULL) && (strlen(album) > 0))
+    {
+        args[pos++] = "--album";
+        args[pos++] = album;
+    }
+
+    if ((genre != NULL) && (strlen(genre) > 0))
+    {
+        args[pos++] = "--genre";
+        args[pos++] = genre;
+    }
+
+    if ((year != NULL) && (strlen(year) > 0))
+    {
+        args[pos++] = "--date";
+        args[pos++] = year;
+    }
+
+    args[pos++] = "-o";
     args[pos++] = aacfilename;
+
+    args[pos++] = wavfilename;
+
     args[pos++] = NULL;
-    
-    fd = exec_with_output(args, STDERR_FILENO, &aac_pid, NULL);
-    
+
+    fd = exec_with_output(args, STDERR_FILENO, &fdkaac_pid, NULL);
+    free(track);
+
     int size;
     char buf[256];
+
     do
     {
-        /* The Nero encoder doesn't give me an estimate for completion
-        * or any way to estimate it myself, just the number of seconds
-        * done. So just sit in here until the program exits */
-        size = read(fd, &buf[0], 256);
-        
-        if (size == -1 && errno == EINTR)
-        /* signal interrupted read(), try again */
-            size = 1;
+        pos = -1;
+        bool interrupted;
+        do
+        {
+            interrupted = FALSE;
+
+            pos++;
+            size = read(fd, &buf[pos], 1);
+
+            if (size == -1 && errno == EINTR)
+            /* signal interrupted read(), try again */
+            {
+                pos--;
+                debugLog("fdkaac() interrupted");
+                interrupted = TRUE;
+            }
+
+        } while ((size > 0 && pos < 255 && buf[pos] != '\r' && buf[pos] != '\n') || interrupted);
+
+        buf[pos] = '\0';
+
+        /* We get lines like this:
+            [28%] 00:14.037/00:49.771 (22x), ETA 00:01.599
+        */
+        if ((pos >= 4) && (buf[0] == '[')) {
+            int percent = 0;
+            if (sscanf(buf, "[%d%%]", &percent) == 1)
+            {
+                *progress = (double) percent / 100.0;
+            }
+        }
     } while (size > 0);
-    
+
     close(fd);
+
     /* don't go on until the signal for the previous call is handled */
-    while (aac_pid != 0)
-    {
-        debugLog("w11\n");
-        usleep(100000);
-    }
-
-    /* Now add tags to the encoded track */
-
-    pos = 0;
-    args[pos++] = "neroAacTag";
-
-    args[pos++] = aacfilename;
-
-    dyn_pos = 0;
-    if (asprintf(&dynamic_args[dyn_pos], "-meta:artist=%s", artist) > 0)
-    {
-        args[pos++] = dynamic_args[dyn_pos++];
-    }
-    if (asprintf(&dynamic_args[dyn_pos], "-meta:title=%s", title) > 0)
-    {
-        args[pos++] = dynamic_args[dyn_pos++];
-    }
-    if (asprintf(&dynamic_args[dyn_pos], "-meta:album=%s", album) > 0)
-    {
-        args[pos++] = dynamic_args[dyn_pos++];
-    }
-    if (asprintf(&dynamic_args[dyn_pos], "-meta:year=%s", year) > 0)
-    {
-        args[pos++] = dynamic_args[dyn_pos++];
-    }
-    if (asprintf(&dynamic_args[dyn_pos], "-meta:genre=%s", genre) > 0)
-    {
-        args[pos++] = dynamic_args[dyn_pos++];
-    }
-    if (asprintf(&dynamic_args[dyn_pos], "-meta:track=%d", tracknum) > 0)
-    {
-        args[pos++] = dynamic_args[dyn_pos++];
-    }
-
-    args[pos++] = NULL;
-    
-    fd = exec_with_output(args, STDERR_FILENO, &aac_pid, NULL);
-    
-    do
-    {
-        /* The Nero tag writer doesn't take very long to run. Just slurp the output. */
-        size = read(fd, &buf[0], 256);
-        
-        if (size == -1 && errno == EINTR)
-        /* signal interrupted read(), try again */
-            size = 1;
-    } while (size > 0);
-    
-    close(fd);
-    /* don't go on until the signal for the previous call is handled */
-    while (aac_pid != 0)
+    while (fdkaac_pid != 0)
     {
         debugLog("w12\n");
         usleep(100000);
     }
 
-    while(dyn_pos)
-    {
-        free(dynamic_args[--dyn_pos]);
-    }
     *progress = 1;
 }
